#!/bin/bash

# Création de liens pour les modules eSBL
# dans le cadre d'une installation
# ou d'une migration
# les données ont normalement été copiées dans
# /home
# Pour la bonne marche des services, il faut
# rétablir, si besoin, les liens vers /home
. /usr/share/eole/FonctionsEoleNg

VERBOSE=""

PERSISTENT_PARTITION="/home"
COMPAT_FOLDER="/data"
FILES_OWNERSHIP="${PERSISTENT_PARTITION}/files_ownership"

SERVICES=( arkwui arkeia apache2 mysql postgresql-8.4 )
declare -a FOLDERS

msg_debug () {
	if [ "$VERBOSE" = "TRUE" ];then echo "$1";fi
}

msg_info () {
	EchoGras "$1"
}

msg_err () {
	EchoRouge "$1"
}

msg_warn () {
	EchoOrange "$1"
}

is_lock () {
	lsof "$1"
	if [ $? -eq 0 ]
	then
            return 0
    else
	    return 1
    fi
}

is_service () {
	if [ -e "/etc/init.d/$1" ]
	then
		return 0
	else
		return 1
	fi
}

stop_services () {
	msg_info "Arrêt des services"
	services_list=( ${SERVICES[*]} )
	for ((i=0; i<=${#services_list[@]}-1; i++))
	do
		is_service "${services_list[$i]}"
		if [ $? -eq 0 ]
		then
			service ${services_list[$i]} stop
		else
			unset SERVICES[$i]
			echo "${SERVICES}"
		fi
	done
}

start_services () {
	msg_info "Démarrage des services"
	services_list=( ${SERVICES[*]} )
	for ((i=${#services_list[@]}-1; i>=0; i--))
	do
		is_service "${services_list[$i]}"
		if [ $? -eq 0 ]
		then
			service ${services_list[$i]} start
		else
			unset SERVICES[$i]
		fi
	done
		
}

create_folder () {
    msg_debug "Création du répertoire $1"
    if [ ! -d "$1" ]
    then
	    /bin/mkdir -p "$1"
    else
	    msg_warn "Le répertoire $1 existe déjà"
    fi
}

move_data () {
    msg_debug "Synchronisation des données de $1 vers $2"
    rsync -a $1 $2 && rm -rf $1
}

create_link () {
    msg_debug "Création du lien symbolique"
    if [ ! -L "$1" ]
    then
	    /bin/ln -s "${DEST_FOLDER}$1" "$1"
    else
	    msg_warn "Le lien $1 existe déjà"
    fi
}

list_ownership () {
	find "$1" -exec stat -c "%U:%G %n" \{\} \; >> "${FILES_OWNERSHIP}"
}

move_to_persistent_partition () {

    if [ "${COMPAT_FOLDER}" = "$1" ]; then
        DEST_FOLDER="${PERSISTENT_PARTITION}"
    else
        DEST_FOLDER="${PERSISTENT_PARTITION}${COMPAT_FOLDER}"
    fi
    msg_info "Déplacement des données de $1 vers $DEST_FOLDER"

    location=$(/bin/readlink -e "$1")
    if [[ ! ${location} == ${PERSISTENT_PARTITION}* ]]
    then
	   if [[ -d "$1" ]]
	   then
		   if [[ ! $(is_lock "$1") ]]
    		   then
			if [ "$2" = "ownership" ]; then
				list_ownership "$1"
			fi
				create_folder $(dirname "${DEST_FOLDER}$1")
				move_data $1 $(dirname "${DEST_FOLDER}$1")
				create_link "$1"
    	           else
	           	msg_err "Le dossier $1 est verrouillé"
			FOLDERS+="$1"
		   fi
	    else
	    msg_warn "Le dossier $1 n'existe pas"
    	    fi
    else
            msg_warn "Le dossier $1 pointe déjà sur $location"
    fi
}

migration_report () {
	if [ ${#FOLDERS} -gt 0 ]
	then
		EchoRouge "Les dossiers suivants n'ont pas été migrés :"
		EchoRouge ${FOLDERS[*]}
	else
		EchoVert "Les dossiers demandés ont été migrés"
	fi
}

lock_quotacheck () {
	Question_ouinon "Voulez-vous empêcher l'exécution de quotacheck après réinstallation ?"
	if [ "$?" -eq 0 ]
	then
		touch /home/quotacheck.lock
	fi
}

backup_acl () {
	Question_ouinon "Voulez-vous effectuer une sauvegarde locale des acl du répertoire bureautique ?"
	if [ "$?" -eq 0 ]
	then
		getfacl -R /bureautique > /home/acl.backup
	fi
}

if [ $(/bin/mount | /bin/grep " ${PERSISTENT_PARTITION} "| /usr/bin/wc -l) -eq 1 ]; then

    backup_acl
    stop_services "$SERVICES"

    move_to_persistent_partition "${COMPAT_FOLDER}"
    move_to_persistent_partition /bureautique
    move_to_persistent_partition /var/www/html "ownership"
    move_to_persistent_partition /var/lib/mysql "ownership"
    move_to_persistent_partition /var/lib/postgresql "ownership"
    move_to_persistent_partition /opt/arkeia
    if [ -e "${FILES_OWNERSHIP}" ]; then
        chmod 400 "${FILES_OWNERSHIP}"
    fi

    start_services "$SERVICES"
    migration_report
    lock_quotacheck
fi

